﻿using SourceChord.FluentWPF;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using VideoImageBuilder.Service;
using VideoImageBuilder.SubPage;
using static System.Net.Mime.MediaTypeNames;

namespace VideoImageBuilder
{
    /// <summary>
    /// MainWindow.xaml 的互動邏輯
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            //this.serviceProvider = serviceProvider;
            //this.appSettingService = appSettingService;
            this.appSettingService = AppSettingService.Instance;

            appSettingService.LoadSetting();

            InitializeComponent();

            uxTitle.Text = uxTitle.Text + string.Format(" V{0}", Assembly.GetEntryAssembly().GetName().Version);

            uxChargeBoxSettingViewItem.IsSelected = true;
            uxChargeBoxSettingViewItem.Focus();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            appSettingService.SaveSetting();
            base.OnClosing(e);
        }

        private ListViewItem lastSelectedItem = null;
        //private readonly IServiceProvider serviceProvider;
        private readonly AppSettingService appSettingService;

        private void uxMenuListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!(sender is ListView listView) ||
                listView.SelectedItem is null)
            {
                return;
            }

            if (lastSelectedItem != null)
            {
                lastSelectedItem.IsSelected = false;
                lastSelectedItem.Foreground = new SolidColorBrush(Colors.Black);
                lastSelectedItem.Background = null;
            }

            ListViewItem item = listView.SelectedItem as ListViewItem;

            Binding myBinding = new Binding("ImmersiveSystemAccentBrush");
            myBinding.Source = new AccentColors();
            item.SetBinding(ListViewItem.ForegroundProperty, myBinding);
            item.Background = new SolidColorBrush(Color.FromArgb(40, 0, 0, 0));

            uxTitleTextBlock.Text = (item.Content as StackPanel).Children.OfType<TextBlock>().First().Text;

            uxContentGrid.Children.Clear();

            //switch (item.Name)
            //{
            //}

            uxContentGrid.Children.Add(GetSubPage(item.Name));

            lastSelectedItem = item;
        }

        private UserControl GetSubPage(string name)
        {
            UserControl subPage = null;
            switch(name)
            {
                case "uxChargeBoxSettingViewItem":
                    subPage = new TargetChargeBoxUc();
                    break;
                case "uxVideoSelectViewItem":
                    subPage = new VideoSelectUc();
                    break;
                case "uxImageSelectViewItem":
                    subPage = new ImageSelectUc();
                    break;
                case "uxGenerateImageViewItem":
                    subPage = new GenerateImageUc();
                    break;
                default:
                    subPage = null;
                    break;
            }

            if(subPage is null)
            {
                return new UserControl();
            }

            return subPage;
        }
    }
}
