﻿using Zerova_EVSE_UI_Tool;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace VideoImageBuilder.Service
{
    public class ImageBuilerService
    {
        public static readonly string ZIP_SOURCE_DIRECTORY = Environment.CurrentDirectory + @"\ZipSource";
        public static readonly string ZIP_FILE_DIRECTORY = Environment.CurrentDirectory + @"\ZipFile";
        public const string TEMP_ZIP_FILE_NAME = @"tmp.zip";

        public const string ZIP_Video1_Name = "97_ScreenSave.mp4";
        public const string ZIP_Video2_Name = "99_AD.mp4";
        public const string ZIP_Image1_Name = "300_SelectGun";

        public Result Generate(
            string chargeboxId,
            string videoPath1,
            string videoPath2,
            string imagePath1,
            string outputFolderPath
            )
        {
            var initResult = InitDirectory();
            if (!initResult)
            {
                return new Result() { Success = false, Message = "InitDirectory Failed" };
            }

            string temZipPath = Path.Combine(ZIP_FILE_DIRECTORY, TEMP_ZIP_FILE_NAME);
            var createZipFileResult = CreateZipFile(temZipPath, videoPath1, videoPath2, imagePath1);
            if (!createZipFileResult)
            {
                return new Result() { Success = false, Message = "CreateZipFile Failed" };
            }

            var createHeaderZipFileResult = CreateHeaderZipFile(outputFolderPath, temZipPath, chargeboxId);
            if (!createHeaderZipFileResult)
            {
                return new Result() { Success = false, Message = "CreateHeaderZipFile Failed" };
            }

            return new Result() { Success = true };
        }

        private static bool CreateHeaderZipFile(string outputFolderPath, string temZipPath, string chargeboxId)
        {
            try
            {

                string zipFileName = string.Format("Image_{0}_{1}.zip", chargeboxId, DateTime.Now.ToString("yyyyMMddHHmmss"));
                string tempHeaderBuildedZipPath = Path.Combine(ZIP_FILE_DIRECTORY, zipFileName);
                FirmwareHeaderBuilder.AddFirmwareHeader(temZipPath, chargeboxId, tempHeaderBuildedZipPath);
                File.Copy(tempHeaderBuildedZipPath, Path.Combine(outputFolderPath, zipFileName));
                return true;
            }
            catch
            {
                return false;
            }
        }

        private static bool CreateZipFile(string zipPath, string videoPath1, string videoPath2, string imagePath1)
        {
            try
            {
                if (!string.IsNullOrEmpty(videoPath1) && File.Exists(videoPath1))
                {
                    File.Copy(videoPath1, Path.Combine(ZIP_SOURCE_DIRECTORY, ZIP_Video1_Name), true);
                }

                if (!string.IsNullOrEmpty(videoPath2) && File.Exists(videoPath2))
                {
                    File.Copy(videoPath2, Path.Combine(ZIP_SOURCE_DIRECTORY, ZIP_Video2_Name), true);
                }

                if (!string.IsNullOrEmpty(imagePath1) && File.Exists(imagePath1))
                {
                    var extention = Path.GetExtension(imagePath1);
                    File.Copy(imagePath1, Path.Combine(ZIP_SOURCE_DIRECTORY, ZIP_Image1_Name + extention), true);
                }

                ZipFile.CreateFromDirectory(ZIP_SOURCE_DIRECTORY, zipPath, CompressionLevel.Fastest, false);
                return true;
            }
            catch
            {
                return false;
            }
        }

        private static bool InitDirectory()
        {
            try
            {
                if (Directory.Exists(ZIP_SOURCE_DIRECTORY))
                {
                    Directory.Delete(ZIP_SOURCE_DIRECTORY, true);
                }
                if (Directory.Exists(ZIP_FILE_DIRECTORY))
                {
                    Directory.Delete(ZIP_FILE_DIRECTORY, true);
                }
                Directory.CreateDirectory(ZIP_SOURCE_DIRECTORY);
                Directory.CreateDirectory(ZIP_FILE_DIRECTORY);
                return true;
            }
            catch
            {
                return false;
            }
        }
    }

    public class Result
    {
        public bool Success { get; set; }
        public string Message { get; set; }
    }
}
