﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace VideoImageBuilder.Service
{
    public class UiGenerateImageService
    {
        public UiGenerateImageService()
        {
            this.uiContext = ImageBuilderContext.Instance;
            this.videoImageBuilerService = new ImageBuilerService();
        }

        private readonly ImageBuilderContext uiContext;
        private readonly ImageBuilerService videoImageBuilerService;

        internal void Start()
        {
            if (!CheckContext())
            {
                return;
            }

            Result result = videoImageBuilerService.Generate(
                uiContext.ChargeboxId,
                uiContext.VideoPath1,
                uiContext.VideoPath2,
                uiContext.ImagePath1,
                uiContext.OutPutFolderPath
                );

            if (result == null)
            {
                MessageBox.Show("Unexpected Error!");
                return;
            }

            if (!result.Success)
            {
                MessageBox.Show(result.Message);
                return;
            }

            System.Diagnostics.Process process = new System.Diagnostics.Process();
            process.StartInfo.FileName = uiContext.OutPutFolderPath ;
            process.Start();
        }

        private bool CheckContext()
        {
            if (string.IsNullOrEmpty(uiContext.ChargeboxId))
            {
                MessageBox.Show("ChargeboxId shoud not be empty");
                return false;
            }
            if (string.IsNullOrEmpty(uiContext.VideoPath1) && 
                string.IsNullOrEmpty(uiContext.VideoPath2) &&
                string.IsNullOrEmpty(uiContext.ImagePath1))
            {
                MessageBox.Show("at least one update is required");
                return false;
            }
            if (!string.IsNullOrEmpty(uiContext.VideoPath1) && !File.Exists(uiContext.VideoPath1))
            {
                MessageBox.Show("Vertical screensave video file not found");
                return false;
            }
            if (!string.IsNullOrEmpty(uiContext.VideoPath2) && !File.Exists(uiContext.VideoPath2))
            {
                MessageBox.Show("Horizontal AD video file not found");
                return false;
            }
            if (!string.IsNullOrEmpty(uiContext.ImagePath1) && !File.Exists(uiContext.ImagePath1))
            {
                MessageBox.Show("Frontpage image file not found");
                return false;
            }
            if (string.IsNullOrEmpty(uiContext.OutPutFolderPath))
            {
                MessageBox.Show("Output folder path should not be empty");
                return false;
            }
            if (!Directory.Exists(uiContext.OutPutFolderPath))
            {
                MessageBox.Show("Output folder not found");
                return false;
            }
            return true;
        }
    }
}
